@@ -1,5 +1,5 @@
 package PAR;
-$PAR::VERSION = '1.002';
+$PAR::VERSION = '1.003';
 
 use 5.006;
 use strict;
@@ -602,7 +602,9 @@ sub _run_member_from_par {
         my $file = $member->fileName;
         print $fh "package main;\n";
         print $fh "#line 1 \"$file\"\n";
+        Archive::Zip::setErrorHandler(sub { die $_[0] });
         $member->extractToFileHandle($fh);
+        Archive::Zip::setErrorHandler(undef);
         seek ($fh, 0, 0);
     }
 
@@ -626,7 +628,9 @@ sub _run_member {
             print $fh "Internals::PAR::CLEARSTACK();\n";
         }
         print $fh "#line 1 \"$file\"\n";
+        Archive::Zip::setErrorHandler(sub { die $_[0] });
         $member->extractToFileHandle($fh);
+        Archive::Zip::setErrorHandler(undef);
         seek ($fh, 0, 0);
     }
 
@@ -712,7 +716,7 @@ sub _extract_inc {
 
             $zip = Archive::Zip->new;
             ( $zip->readFromFileHandle($fh, $file_or_azip_handle) == Archive::Zip::AZ_OK() )
-                or die "Read '$file_or_azip_handle' error: $!";
+                or die "Error reading '$file_or_azip_handle': $!";
           }
           else {
             $zip = $file_or_azip_handle;
@@ -720,6 +724,7 @@ sub _extract_inc {
 
           mkdir($inc) if not -d $inc;
 
+          Archive::Zip::setErrorHandler(sub { die $_[0] });
           for ( $zip->memberNames() ) {
               s{^/}{};
 
@@ -731,6 +736,7 @@ sub _extract_inc {
               next if -e $outfile and not -w _;
               $zip->extractMember($_, "$inc/" . $_);
           }
+          Archive::Zip::setErrorHandler(undef);
         }
         
         rmdir("$inc.lock");
@@ -1008,6 +1014,7 @@ sub unpar {
             my @members = _cached_members_matching( $zip,
               qr#^shlib/$Config{archname}/.*\.\Q$Config{dlext}\E(?:\.|$)#
             );
+            Archive::Zip::setErrorHandler(sub { die $_[0] });
             foreach my $member (@members) {
                 next if $member->isDirectory;
                 my $member_name = $member->fileName;
@@ -1024,6 +1031,7 @@ sub unpar {
                 $member->extractToFileNamed($dest_name)
                     unless(-e $dest_name);
             }
+            Archive::Zip::setErrorHandler(undef);
         }
 
         # Now push this path into usual library search paths
@@ -1070,7 +1078,9 @@ sub unpar {
     die "Bad Things Happened..." unless $fh;
 
     if ($is_new) {
+        Archive::Zip::setErrorHandler(sub { die $_[0] });
         $member->extractToFileHandle($fh);
+        Archive::Zip::setErrorHandler(undef);
         seek ($fh, 0, 0);
     }
 
